@@ -108,15 +108,19 @@ class ViewerMixin:
         """Return the currently selected commit object IDs"""
         return [i.commit for i in self.selected_items()]
 
-    def clicked_oid(self):
+    def clicked_oid(self, filtered=True):
         """Return the clicked or selected commit object ID"""
         if self.clicked:
-            return self.clicked.oid
-        return self.selected_oid()
+            oid = self.clicked.oid
+        else:
+            oid = self.selected_oid()
+        if filtered and oid and oid in (dag.STAGE, dag.WORKTREE):
+            oid = None
+        return oid
 
-    def with_oid(self, func):
+    def with_oid(self, func, filtered=True):
         """Run an operation with a commit object ID"""
-        oid = self.clicked_oid()
+        oid = self.clicked_oid(filtered=filtered)
         if oid:
             result = func(oid)
         else:
@@ -203,11 +207,8 @@ class ViewerMixin:
 
     def show_diff(self):
         """Show the diff for the selected commit"""
-        context = self.context
         self.with_oid(
-            lambda oid: difftool.diff_expression(
-                context, self, oid + '^!', hide_expr=False, focus_tree=True
-            )
+            lambda oid: _diff_expression(self.context, self, oid), filtered=False
         )
 
     def show_dir_diff(self):
@@ -215,8 +216,13 @@ class ViewerMixin:
         context = self.context
         self.with_oid(
             lambda oid: difftool.difftool_launch(
-                context, left=oid, left_take_magic=True, dir_diff=True
-            )
+                context,
+                left=oid,
+                left_take_magic=True,
+                dir_diff=True,
+                staged=oid == dag.STAGE,
+            ),
+            filtered=False,
         )
 
     def rebase_to_commit(self):
@@ -262,12 +268,15 @@ class ViewerMixin:
     def save_blob_dialog(self):
         """Save a file blob from the selected commit"""
         context = self.context
-        self.with_oid(lambda oid: browse.BrowseBranch.browse(context, oid))
+        self.with_oid(
+            lambda oid: browse.BrowseBranch.browse(context, oid), filtered=False
+        )
 
     def save_blob_from_parent_dialog(self):
         """Save a file blob from the parent of the selected commit"""
-        context = self.context
-        self.with_oid(lambda oid: browse.BrowseBranch.browse(context, oid + '^'))
+        self.with_oid(
+            lambda oid: _save_blob_from_parent(self.context, oid), filtered=False
+        )
 
     def update_menu_actions(self, event):
         """Update menu actions to reflect the selection state"""
@@ -279,6 +288,7 @@ class ViewerMixin:
         else:
             self.clicked = commit = item.commit
 
+        has_oid = commit and commit.oid not in (dag.WORKTREE, dag.STAGE)
         has_single_selection = len(selected_items) == 1
         has_single_selection_or_clicked = bool(has_single_selection or commit)
         has_selection = bool(selected_items)
@@ -303,31 +313,56 @@ class ViewerMixin:
         self.menu_actions['diff_selected_this'].setEnabled(can_diff)
         self.menu_actions['diff_commit'].setEnabled(has_single_selection_or_clicked)
         self.menu_actions['diff_commit_all'].setEnabled(has_single_selection_or_clicked)
-
-        self.menu_actions['checkout_branch'].setEnabled(has_branches)
+        self.menu_actions['checkout_branch'].setEnabled(bool(has_branches) and has_oid)
         self.menu_actions['checkout_detached'].setEnabled(
-            has_single_selection_or_clicked
+            has_single_selection_or_clicked and has_oid
+        )
+        self.menu_actions['cherry_pick'].setEnabled(
+            has_single_selection_or_clicked and has_oid
+        )
+        self.menu_actions['copy'].setEnabled(
+            has_single_selection_or_clicked and has_oid
+        )
+        self.menu_actions['copy_short'].setEnabled(
+            has_single_selection_or_clicked and has_oid
+        )
+        self.menu_actions['create_branch'].setEnabled(
+            has_single_selection_or_clicked and has_oid
+        )
+        self.menu_actions['create_patch'].setEnabled(has_selection and has_oid)
+        self.menu_actions['create_tag'].setEnabled(
+            has_single_selection_or_clicked and has_oid
+        )
+        self.menu_actions['create_tarball'].setEnabled(
+            has_single_selection_or_clicked and has_oid
         )
-        self.menu_actions['cherry_pick'].setEnabled(has_single_selection_or_clicked)
-        self.menu_actions['copy'].setEnabled(has_single_selection_or_clicked)
-        self.menu_actions['copy_short'].setEnabled(has_single_selection_or_clicked)
-        self.menu_actions['create_branch'].setEnabled(has_single_selection_or_clicked)
-        self.menu_actions['create_patch'].setEnabled(has_selection)
-        self.menu_actions['create_tag'].setEnabled(has_single_selection_or_clicked)
-        self.menu_actions['create_tarball'].setEnabled(has_single_selection_or_clicked)
         self.menu_actions['rebase_to_commit'].setEnabled(
-            has_single_selection_or_clicked
+            has_single_selection_or_clicked and has_oid
+        )
+        self.menu_actions['reset_mixed'].setEnabled(
+            has_single_selection_or_clicked and has_oid
+        )
+        self.menu_actions['reset_keep'].setEnabled(
+            has_single_selection_or_clicked and has_oid
+        )
+        self.menu_actions['reset_merge'].setEnabled(
+            has_single_selection_or_clicked and has_oid
+        )
+        self.menu_actions['reset_soft'].setEnabled(
+            has_single_selection_or_clicked and has_oid
+        )
+        self.menu_actions['reset_hard'].setEnabled(
+            has_single_selection_or_clicked and has_oid
         )
-        self.menu_actions['reset_mixed'].setEnabled(has_single_selection_or_clicked)
-        self.menu_actions['reset_keep'].setEnabled(has_single_selection_or_clicked)
-        self.menu_actions['reset_merge'].setEnabled(has_single_selection_or_clicked)
-        self.menu_actions['reset_soft'].setEnabled(has_single_selection_or_clicked)
-        self.menu_actions['reset_hard'].setEnabled(has_single_selection_or_clicked)
         self.menu_actions['restore_worktree'].setEnabled(
-            has_single_selection_or_clicked
+            has_single_selection_or_clicked and has_oid
+        )
+        self.menu_actions['revert'].setEnabled(
+            has_single_selection_or_clicked and has_oid
+        )
+        self.menu_actions['save_blob'].setEnabled(
+            has_single_selection_or_clicked and has_oid
         )
-        self.menu_actions['revert'].setEnabled(has_single_selection_or_clicked)
-        self.menu_actions['save_blob'].setEnabled(has_single_selection_or_clicked)
         self.menu_actions['save_blob_from_parent'].setEnabled(
             has_single_selection_or_clicked
         )
@@ -368,56 +403,78 @@ class ViewerMixin:
         menu.exec_(self.mapToGlobal(event.pos()))
 
 
+def _diff_expression(context, widget, oid):
+    """Launch difftool using the specified object ID"""
+    if oid == dag.WORKTREE:
+        ref = ''
+    elif oid == dag.STAGE:
+        ref = '--cached'
+    else:
+        ref = f'{oid}^!'
+    return difftool.diff_expression(
+        context, widget, ref, hide_expr=False, focus_tree=True
+    )
+
+
+def _save_blob_from_parent(context, oid):
+    """Save a browse dialog to grab a file from the parent commit"""
+    if oid in (dag.STAGE, dag.WORKTREE):
+        ref = 'HEAD'
+    else:
+        ref = f'{oid}^'
+    return browse.BrowseBranch.browse(context, ref)
+
+
 def set_icon(icon, action):
     """ "Set the icon for an action and return the action"""
     action.setIcon(icon)
     return action
 
 
-def viewer_actions(widget):
+def viewer_actions(widget, proxy):
     """Return common actions across the tree and graph widgets"""
     return {
         'diff_this_selected': set_icon(
             icons.compare(),
             qtutils.add_action(
-                widget, N_('Diff this -> selected'), widget.proxy.diff_this_selected
+                widget, N_('Diff this -> selected'), proxy.diff_this_selected
             ),
         ),
         'diff_selected_this': set_icon(
             icons.compare(),
             qtutils.add_action(
-                widget, N_('Diff selected -> this'), widget.proxy.diff_selected_this
+                widget, N_('Diff selected -> this'), proxy.diff_selected_this
             ),
         ),
         'create_branch': set_icon(
             icons.branch(),
-            qtutils.add_action(widget, N_('Create Branch'), widget.proxy.create_branch),
+            qtutils.add_action(widget, N_('Create Branch'), proxy.create_branch),
         ),
         'create_patch': set_icon(
             icons.save(),
-            qtutils.add_action(widget, N_('Create Patch'), widget.proxy.create_patch),
+            qtutils.add_action(widget, N_('Create Patch'), proxy.create_patch),
         ),
         'create_tag': set_icon(
             icons.tag(),
-            qtutils.add_action(widget, N_('Create Tag'), widget.proxy.create_tag),
+            qtutils.add_action(widget, N_('Create Tag'), proxy.create_tag),
         ),
         'create_tarball': set_icon(
             icons.file_zip(),
             qtutils.add_action(
-                widget, N_('Save As Tarball/Zip...'), widget.proxy.create_tarball
+                widget, N_('Save As Tarball/Zip...'), proxy.create_tarball
             ),
         ),
         'cherry_pick': set_icon(
             icons.cherry_pick(),
-            qtutils.add_action(widget, N_('Cherry Pick'), widget.proxy.cherry_pick),
+            qtutils.add_action(widget, N_('Cherry Pick'), proxy.cherry_pick),
         ),
         'revert': set_icon(
-            icons.undo(), qtutils.add_action(widget, N_('Revert'), widget.proxy.revert)
+            icons.undo(), qtutils.add_action(widget, N_('Revert'), proxy.revert)
         ),
         'diff_commit': set_icon(
             icons.diff(),
             qtutils.add_action(
-                widget, N_('Launch Diff Tool'), widget.proxy.show_diff, hotkeys.DIFF
+                widget, N_('Launch Diff Tool'), proxy.show_diff, hotkeys.DIFF
             ),
         ),
         'diff_commit_all': set_icon(
@@ -425,35 +482,31 @@ def viewer_actions(widget):
             qtutils.add_action(
                 widget,
                 N_('Launch Directory Diff Tool'),
-                widget.proxy.show_dir_diff,
+                proxy.show_dir_diff,
                 hotkeys.DIFF_SECONDARY,
             ),
         ),
         'checkout_branch': set_icon(
             icons.branch(),
-            qtutils.add_action(
-                widget, N_('Checkout Branch'), widget.proxy.checkout_branch
-            ),
+            qtutils.add_action(widget, N_('Checkout Branch'), proxy.checkout_branch),
         ),
         'checkout_detached': qtutils.add_action(
-            widget, N_('Checkout Detached HEAD'), widget.proxy.checkout_detached
+            widget, N_('Checkout Detached HEAD'), proxy.checkout_detached
         ),
         'rebase_to_commit': set_icon(
             icons.play(),
             qtutils.add_action(
-                widget, N_('Rebase to this commit'), widget.proxy.rebase_to_commit
+                widget, N_('Rebase to this commit'), proxy.rebase_to_commit
             ),
         ),
         'reset_soft': set_icon(
             icons.style_dialog_reset(),
-            qtutils.add_action(
-                widget, N_('Reset Branch (Soft)'), widget.proxy.reset_soft
-            ),
+            qtutils.add_action(widget, N_('Reset Branch (Soft)'), proxy.reset_soft),
         ),
         'reset_mixed': set_icon(
             icons.style_dialog_reset(),
             qtutils.add_action(
-                widget, N_('Reset Branch and Stage (Mixed)'), widget.proxy.reset_mixed
+                widget, N_('Reset Branch and Stage (Mixed)'), proxy.reset_mixed
             ),
         ),
         'reset_keep': set_icon(
@@ -461,7 +514,7 @@ def viewer_actions(widget):
             qtutils.add_action(
                 widget,
                 N_('Restore Worktree and Reset All (Keep Unstaged Edits)'),
-                widget.proxy.reset_keep,
+                proxy.reset_keep,
             ),
         ),
         'reset_merge': set_icon(
@@ -469,7 +522,7 @@ def viewer_actions(widget):
             qtutils.add_action(
                 widget,
                 N_('Restore Worktree and Reset All (Merge)'),
-                widget.proxy.reset_merge,
+                proxy.reset_merge,
             ),
         ),
         'reset_hard': set_icon(
@@ -477,27 +530,23 @@ def viewer_actions(widget):
             qtutils.add_action(
                 widget,
                 N_('Restore Worktree and Reset All (Hard)'),
-                widget.proxy.reset_hard,
+                proxy.reset_hard,
             ),
         ),
         'restore_worktree': set_icon(
             icons.edit(),
-            qtutils.add_action(
-                widget, N_('Restore Worktree'), widget.proxy.restore_worktree
-            ),
+            qtutils.add_action(widget, N_('Restore Worktree'), proxy.restore_worktree),
         ),
         'save_blob': set_icon(
             icons.save(),
-            qtutils.add_action(
-                widget, N_('Grab File...'), widget.proxy.save_blob_dialog
-            ),
+            qtutils.add_action(widget, N_('Grab File...'), proxy.save_blob_dialog),
         ),
         'save_blob_from_parent': set_icon(
             icons.save(),
             qtutils.add_action(
                 widget,
                 N_('Grab File from Parent Commit...'),
-                widget.proxy.save_blob_from_parent_dialog,
+                proxy.save_blob_from_parent_dialog,
             ),
         ),
         'copy': set_icon(
@@ -505,7 +554,7 @@ def viewer_actions(widget):
             qtutils.add_action(
                 widget,
                 N_('Copy Commit'),
-                widget.proxy.copy_to_clipboard,
+                proxy.copy_to_clipboard,
                 hotkeys.COPY_COMMIT_ID,
             ),
         ),
@@ -514,7 +563,7 @@ def viewer_actions(widget):
             qtutils.add_action(
                 widget,
                 N_('Copy Commit (Short)'),
-                widget.proxy.copy_to_clipboard_short,
+                proxy.copy_to_clipboard_short,
                 hotkeys.COPY,
             ),
         ),
@@ -612,13 +661,14 @@ class CommitTreeWidget(standard.TreeWidget, ViewerMixin):
 
         self.setSelectionMode(self.ExtendedSelection)
         self.setHeaderLabels([N_('Summary'), N_('Author'), N_('Date, Time')])
+        self.header().setSectionResizeMode(2, QtWidgets.QHeaderView.Stretch)
 
         self.context = context
         self.oidmap = {}
         self.menu_actions = None
         self.selecting = False
         self.commits = []
-        self._adjust_columns = False
+        self._columns_initialized = False
         self.action_up = qtutils.add_action(
             self, N_('Go Up'), self.go_up, hotkeys.MOVE_UP
         )
@@ -648,29 +698,25 @@ class CommitTreeWidget(standard.TreeWidget, ViewerMixin):
         except (KeyError, ValueError):
             column_widths = None
         if column_widths:
-            self.set_column_widths(column_widths)
-        else:
-            # Defer showing the columns until we are shown, and our true width
-            # is known.  Calling adjust_columns() here ends up with the wrong
-            # answer because we have not yet been parented to the layout.
-            # We set this flag that we process once during our initial
-            # showEvent().
-            self._adjust_columns = True
+            # We only care about the first two columns. This allows the final
+            # column to stretch and shrink.
+            self.set_column_widths(column_widths[:2])
+            self._columns_initialized = True
         return True
 
     # Qt overrides
     def showEvent(self, event):
         """Override QWidget::showEvent() to size columns when we are shown"""
-        if self._adjust_columns:
-            self._adjust_columns = False
-            width = self.width()
-            two_thirds = (width * 2) // 3
-            one_sixth = width // 6
-
-            self.setColumnWidth(0, two_thirds)
-            self.setColumnWidth(1, one_sixth)
-            self.setColumnWidth(2, one_sixth)
-        return standard.TreeWidget.showEvent(self, event)
+        standard.TreeWidget.showEvent(self, event)
+        # Defer resizing columns until the widget has been shown so that width() returns
+        # the correct value.
+        if not self._columns_initialized:
+            self._columns_initialized = True
+            width = self.header().width()
+            one_half = width // 2
+            one_quarter = width // 4
+            self.setColumnWidth(0, one_half)
+            self.setColumnWidth(1, one_quarter)
 
     # ViewerMixin
     def go_up(self):
@@ -803,6 +849,7 @@ class GitDAG(standard.MainWindow):
         self.old_oids = None
         self.old_count = 0
         self.force_refresh = False
+        self._widgets_initialized = False
 
         self.thread = None
         self.revtext = GitDagLineEdit(context)
@@ -842,23 +889,37 @@ class GitDAG(standard.MainWindow):
             self.treewidget, self.graphview, self.filewidget
         )
 
-        self.viewer_actions = actions = viewer_actions(self)
-        self.treewidget.menu_actions = actions
-        self.graphview.menu_actions = actions
+        self.treewidget.menu_actions = viewer_actions(self.treewidget, self.proxy)
+        self.graphview.menu_actions = viewer_actions(self.graphview, self.proxy)
+        self.diffwidget_copy_commit = set_icon(
+            icons.copy(),
+            qtutils.add_action(
+                self.diffwidget.diff,
+                N_('Copy Commit'),
+                self.treewidget.copy_to_clipboard,
+                hotkeys.COPY_COMMIT_ID,
+            ),
+        )
+        self.diffwidget.diff.menu_actions.append(self.diffwidget_copy_commit)
 
         self.controls_layout = qtutils.hbox(
             defs.no_margin, defs.spacing, self.revtext, self.maxresults
         )
+        self.controls_layout.setAlignment(self.maxresults, Qt.AlignTop)
 
         self.controls_widget = QtWidgets.QWidget()
         self.controls_widget.setLayout(self.controls_layout)
 
-        self.log_dock = qtutils.create_dock('Log', N_('Log'), self, stretch=False)
+        self.log_dock = qtutils.create_dock(
+            'Log', N_('Log'), self, stretch=False, hide_title=True
+        )
         self.log_dock.setWidget(self.treewidget)
         log_dock_titlebar = self.log_dock.titleBarWidget()
         log_dock_titlebar.add_corner_widget(self.controls_widget)
 
-        self.file_dock = qtutils.create_dock('Files', N_('Files'), self)
+        self.file_dock = qtutils.create_dock(
+            'Files', N_('Files'), self, hide_title=True
+        )
         self.file_dock.setWidget(self.filewidget)
 
         self.diff_panel = diff.DiffPanel(self.diffwidget, self.diffwidget.diff, self)
@@ -867,11 +928,11 @@ class GitDAG(standard.MainWindow):
         self.diff_options.hide_advanced_options()
         self.diff_options.set_diff_type(main.Types.TEXT)
 
-        self.diff_dock = qtutils.create_dock('Diff', N_('Diff'), self)
+        self.diff_dock = qtutils.create_dock('Diff', N_('Diff'), self, hide_title=True)
         self.diff_dock.setWidget(self.diff_panel)
 
         diff_titlebar = self.diff_dock.titleBarWidget()
-        diff_titlebar.add_corner_widget(self.diff_options)
+        diff_titlebar.add_title_widget(self.diff_options)
 
         self.graph_controls_layout = qtutils.hbox(
             defs.no_margin,
@@ -885,7 +946,9 @@ class GitDAG(standard.MainWindow):
         self.graph_controls_widget = QtWidgets.QWidget()
         self.graph_controls_widget.setLayout(self.graph_controls_layout)
 
-        self.graphview_dock = qtutils.create_dock('Graph', N_('Graph'), self)
+        self.graphview_dock = qtutils.create_dock(
+            'Graph', N_('Graph'), self, hide_title=True
+        )
         self.graphview_dock.setWidget(self.graphview)
         graph_titlebar = self.graphview_dock.titleBarWidget()
         graph_titlebar.add_corner_widget(self.graph_controls_widget)
@@ -950,7 +1013,6 @@ class GitDAG(standard.MainWindow):
     def set_params(self, params):
         context = self.context
         self.params = params
-
         # Update fields affected by model
         self.revtext.setText(params.ref)
         self.maxresults.setValue(params.count)
@@ -958,10 +1020,7 @@ class GitDAG(standard.MainWindow):
 
         if self.thread is not None:
             self.thread.stop()
-
-        self.thread = ReaderThread(context, params, self)
-
-        thread = self.thread
+        self.thread = thread = ReaderThread(context, params, self)
         thread.begin.connect(self.thread_begin, type=Qt.QueuedConnection)
         thread.status.connect(self.thread_status, type=Qt.QueuedConnection)
         thread.add.connect(self.add_commits, type=Qt.QueuedConnection)
@@ -1075,6 +1134,8 @@ class GitDAG(standard.MainWindow):
 
     def select_commits(self, commits):
         self.selection = commits
+        enabled = bool(commits)
+        self.diffwidget_copy_commit.setEnabled(enabled)
 
     def clear(self):
         self.commits.clear()
@@ -1127,12 +1188,6 @@ class GitDAG(standard.MainWindow):
         else:
             difftool.diff_commits(self.context, self, left, right)
 
-    # Qt overrides
-    def closeEvent(self, event):
-        self.revtext.close_popup()
-        self.thread.stop()
-        standard.MainWindow.closeEvent(self, event)
-
     def histories_selected(self, histories):
         argv = [self.model.currentbranch, '--']
         argv.extend(histories)
@@ -1160,6 +1215,20 @@ class GitDAG(standard.MainWindow):
         model = browse.BrowseModel(oid, filename=filename)
         browse.save_path(self.context, filename, model)
 
+    # Qt overrides
+    def closeEvent(self, event):
+        """Ensure the revtext popup is closed"""
+        self.revtext.close_popup()
+        self.thread.stop()
+        standard.MainWindow.closeEvent(self, event)
+
+    def showEvent(self, event):
+        """Resize widgets once their sizes are known"""
+        standard.MainWindow.showEvent(self, event)
+        if not self._widgets_initialized:
+            self._widgets_initialized = True
+            self.maxresults.setMinimumHeight(self.revtext.height())
+
 
 class ReaderThread(QtCore.QThread):
     begin = Signal()
@@ -1169,12 +1238,10 @@ class ReaderThread(QtCore.QThread):
 
     def __init__(self, context, params, parent):
         QtCore.QThread.__init__(self, parent)
+        self.setTerminationEnabled(True)
         self.context = context
         self.params = params
-        self._abort = False
         self._stop = False
-        self._mutex = QtCore.QMutex()
-        self._condition = QtCore.QWaitCondition()
 
     def run(self):
         context = self.context
@@ -1183,11 +1250,7 @@ class ReaderThread(QtCore.QThread):
         self.begin.emit()
         commits = []
         for commit in repo.get():
-            self._mutex.lock()
             if self._stop:
-                self._condition.wait(self._mutex)
-            self._mutex.unlock()
-            if self._abort:
                 repo.reset()
                 return
             commits.append(commit)
@@ -1195,30 +1258,24 @@ class ReaderThread(QtCore.QThread):
                 self.add.emit(commits)
                 commits = []
 
-        self.status.emit(repo.returncode == 0)
+        stage, worktree = repo.get_worktree_commits()
+        if stage:
+            commits.append(stage)
+        if worktree:
+            commits.append(worktree)
         if commits:
             self.add.emit(commits)
+
+        self.status.emit(repo.returncode == 0)
         self.end.emit()
 
     def start(self):
-        self._abort = False
         self._stop = False
         QtCore.QThread.start(self)
 
-    def pause(self):
-        self._mutex.lock()
-        self._stop = True
-        self._mutex.unlock()
-
-    def resume(self):
-        self._mutex.lock()
-        self._stop = False
-        self._mutex.unlock()
-        self._condition.wakeOne()
-
     def stop(self):
-        self._abort = True
-        self.wait()
+        self._stop = True
+        self.terminate()
 
 
 class Cache:
